@@ -1,4 +1,5 @@
 <?php
+declare(strict_types=1);
 namespace MRBS;
 
 use MRBS\Form\Form;
@@ -12,7 +13,7 @@ require_once "functions_mail.inc";
 // Get non-standard form variables
 $action = get_form_var('action', 'string');
 $id = get_form_var('id', 'int');
-$series = get_form_var('series', 'int');
+$series = get_form_var('series', 'bool');
 $returl = get_form_var('returl', 'string');
 $note = get_form_var('note', 'string');
 
@@ -33,7 +34,7 @@ $mail_previous = array();
 $start_times = array();
 
 // Give the return URL a query string if it doesn't already have one
-if (utf8_strpos($returl, '?') === false)
+if (mb_strpos($returl, '?') === false)
 {
   $returl .= "?year=$year&month=$month&day=$day&area=$area&room=$room";
 }
