@@ -1,18 +1,19 @@
 <?php
+declare(strict_types=1);
 namespace MRBS;
 
-use MRBS\Form\Form;
+use MRBS\DB\DBException;
 use MRBS\Form\ElementInputSubmit;
+use MRBS\Form\Form;
 
 require "defaultincludes.inc";
 
 
-function generate_no_form($room, $area)
+function generate_no_form(int $room, int $area) : void
 {
-  $form = new Form();
+  $form = new Form(Form::METHOD_POST);
 
-  $attributes = array('action' => multisite('admin.php'),
-                      'method' => 'post');
+  $attributes = array('action' => multisite('admin.php'));
 
   $form->setAttributes($attributes);
 
@@ -30,12 +31,11 @@ function generate_no_form($room, $area)
 }
 
 
-function generate_yes_form($room, $area)
+function generate_yes_form(int $room, int $area) : void
 {
-  $form = new Form();
+  $form = new Form(Form::METHOD_POST);
 
-  $attributes = array('action' => multisite('del.php'),
-                      'method' => 'post');
+  $attributes = array('action' => multisite('del.php'));
 
   $form->setAttributes($attributes);
 
@@ -72,7 +72,7 @@ $context = array(
     'month'     => $month,
     'day'       => $day,
     'area'      => $area,
-    'room'      => isset($room) ? $room : null
+    'room'      => $room ?? null
   );
 
 // This is gonna blast away something. We want them to be really
@@ -114,53 +114,40 @@ if ($type == "room")
 
     // We tell them how bad what they're about to do is
     // Find out how many appointments would be deleted
-    $limit = 20;
+    // Do a quick count of the number of entries
+    $n_entries = get_n_entries_by_room($room);
 
-    $sql = "SELECT COUNT(*)
-              FROM " . _tbl('entry') . "
-             WHERE room_id=?";
-
-    $n_bookings = db()->query1($sql, array($room));
-
-    // The LIMIT parameter should ideally be one of the parameters to the
-    // query, but MySQL throws an error at the moment because it gets bound
-    // as a string.  Doesn't matter in this case because we know where $limit
-    // has come from, but for the general case MRBS needs to provide the ability
-    // to bind it as an integer.
-    //
-    // Order in descending order because the latest bookings are probably the most
-    // important.
-    $sql = "SELECT name, start_time, end_time
-              FROM " . _tbl('entry') . "
-             WHERE room_id=?
-          ORDER BY start_time DESC
-             LIMIT $limit";
-
-    $res = db()->query($sql, array($room));
-
-    if ($res->count() > 0)
+    if ($n_entries > 0)
     {
-      echo "<p>\n";
-      echo get_vocab("deletefollowing") . ":\n";
-      echo "</p>\n";
+      $limit = 20;
+      // Order in descending order because the latest bookings are probably the most important.
+      $entries = get_entries_by_room($room, null, null, true, $limit);
 
-      echo "<ul>\n";
-
-      while (false !== ($row = $res->next_row_keyed()))
+      // We can't rely on ($n_entries > 0) because there's a very small chance the number of entries
+      // may have changed between the two queries
+      if (count($entries) > 0)
       {
-        echo "<li>".htmlspecialchars($row['name'])." (";
-        echo time_date_string($row['start_time']) . " -> ";
-        echo time_date_string($row['end_time']) . ")</li>\n";
+        echo "<p>\n";
+        echo get_vocab("deletefollowing") . ":\n";
+        echo "</p>\n";
+
+        echo "<ul>\n";
+
+        foreach ($entries as $entry)
+        {
+          $interval = new EntryInterval($entry['start_time'], $entry['end_time'], $enable_periods);
+          echo "<li>" . escape_html($entry['name']) . " (" . $interval . ")</li>\n";
+        }
+
+        echo "</ul>\n";
       }
 
-      echo "</ul>\n";
-    }
-
-    if ($n_bookings > $limit)
-    {
-      echo "<p>";
-      echo get_vocab("and_n_more", number_format_locale($n_bookings - $limit)) . '.';
-      echo "</p>";
+      if ($n_entries > $limit)
+      {
+        echo "<p>";
+        echo get_vocab("and_n_more", number_format_locale($n_entries - $limit)) . '.';
+        echo "</p>";
+      }
     }
 
     echo "<div id=\"del_room_confirm\">\n";
@@ -184,7 +171,7 @@ if ($type == "area")
            WHERE area_id=?";
 
   $n = db()->query1($sql, array($area));
-  if ($n == 0)
+  if ($n === 0)
   {
     // OK, nothing there, let's blast it away
     $sql = "DELETE FROM " . _tbl('area') . "
@@ -203,7 +190,7 @@ if ($type == "area")
     echo get_vocab("delarea");
     $query_vars = array('area' => $area);
     $query = http_build_query($query_vars, '', '&');
-    echo '<a href="' . htmlspecialchars(multisite("admin.php?$query")) . '">' . get_vocab('backadmin') . '</a>';
+    echo '<a href="' . escape_html(multisite("admin.php?$query")) . '">' . get_vocab('back') . '</a>';
     echo "</p>\n";
     print_footer();
     exit;
