@@ -1,12 +1,13 @@
 <?php
+declare(strict_types=1);
 namespace MRBS;
 
-use MRBS\Form\Form;
+use MRBS\Errors\Errors;
 use MRBS\Form\ElementDiv;
+use MRBS\Form\ElementFieldset;
 use MRBS\Form\ElementInputCheckbox;
 use MRBS\Form\ElementInputNumber;
 use MRBS\Form\ElementInputSubmit;
-use MRBS\Form\ElementFieldset;
 use MRBS\Form\ElementLegend;
 use MRBS\Form\ElementP;
 use MRBS\Form\ElementSelect;
@@ -15,9 +16,9 @@ use MRBS\Form\FieldButton;
 use MRBS\Form\FieldDiv;
 use MRBS\Form\FieldInputCheckbox;
 use MRBS\Form\FieldInputCheckboxGroup;
-use MRBS\Form\FieldInputRadioGroup;
 use MRBS\Form\FieldInputEmail;
 use MRBS\Form\FieldInputNumber;
+use MRBS\Form\FieldInputRadioGroup;
 use MRBS\Form\FieldInputSubmit;
 use MRBS\Form\FieldInputText;
 use MRBS\Form\FieldInputTime;
@@ -25,11 +26,12 @@ use MRBS\Form\FieldSelect;
 use MRBS\Form\FieldSpan;
 use MRBS\Form\FieldTextarea;
 use MRBS\Form\FieldTimeWithUnits;
+use MRBS\Form\Form;
 
 require "defaultincludes.inc";
 require_once "mrbs_sql.inc";
 
-function get_timezone_options()
+function get_timezone_options() : array
 {
   global $zoneinfo_outlook_compatible;
 
@@ -39,7 +41,7 @@ function get_timezone_options()
 
   foreach ($timezone_identifiers as $value)
   {
-    if (utf8_strpos($value, '/') === FALSE)
+    if (mb_strpos($value, '/') === FALSE)
     {
       // There are some timezone identifiers (eg 'UTC') on some operating
       // systems that don't fit the Continent/City model.   We'll put them
@@ -70,7 +72,7 @@ function get_timezone_options()
 }
 
 
-function get_fieldset_errors(array $errors)
+function get_fieldset_errors(array $errors) : ElementFieldset
 {
   $fieldset = new ElementFieldset();
   $fieldset->addLegend('')
@@ -87,7 +89,7 @@ function get_fieldset_errors(array $errors)
 }
 
 
-function get_fieldset_general(array $data)
+function get_fieldset_general(array $data) : ElementFieldset
 {
   global $timezone, $auth;
 
@@ -131,7 +133,7 @@ function get_fieldset_general(array $data)
     $field->setLabel(get_vocab('custom_html'))
       ->setLabelAttribute('title', get_vocab('custom_html_note'))
       ->setControlAttribute('name', 'custom_html')
-      ->setControlText($data['custom_html']);
+      ->setControlText($data['custom_html'] ?? '');
     $fieldset->addElement($field);
   }
 
@@ -144,11 +146,15 @@ function get_fieldset_general(array $data)
   $fieldset->addElement($field);
 
   // Default type
-  $field = new FieldSelect();
-  $field->setLabel(get_vocab('default_type'))
-        ->setControlAttribute('name', 'area_default_type')
-        ->addSelectOptions(get_type_options(), $data['default_type'], true);
-  $fieldset->addElement($field);
+  $options = get_type_options(false);
+  if (count($options)>0)
+  {
+    $field = new FieldSelect();
+    $field->setLabel(get_vocab('default_type'))
+          ->setControlAttribute('name', 'area_default_type')
+          ->addSelectOptions($options, $data['default_type'], true);
+    $fieldset->addElement($field);
+  }
 
   // Status - Enabled or Disabled
   $options = array('0' => get_vocab('enabled'),
@@ -182,7 +188,7 @@ function get_fieldset_general(array $data)
 }
 
 
-function get_fieldset_times()
+function get_fieldset_times() : ElementFieldset
 {
   global $enable_periods;
   global $morningstarts, $morningstarts_minutes;
@@ -204,7 +210,7 @@ function get_fieldset_times()
        ->setText(' (' . get_vocab('times_only') . ')');
 
   $legend = new ElementLegend();
-  $legend->setText(get_vocab('time_settings'), $text_at_start=true)
+  $legend->setText(get_vocab('time_settings'), true)
          ->addElement($span);
 
   $fieldset->addLegend($legend);
@@ -265,7 +271,7 @@ function get_fieldset_times()
 }
 
 
-function get_fieldset_periods()
+function get_fieldset_periods() : ElementFieldset
 {
   global $enable_periods, $periods;
 
@@ -311,7 +317,7 @@ function get_fieldset_periods()
 }
 
 
-function get_fieldset_create_ahead()
+function get_fieldset_create_ahead() : ElementFieldset
 {
   global $min_create_ahead_secs, $max_create_ahead_secs,
          $min_create_ahead_enabled, $max_create_ahead_enabled;
@@ -343,7 +349,7 @@ function get_fieldset_create_ahead()
 }
 
 
-function get_fieldset_delete_ahead()
+function get_fieldset_delete_ahead() : ElementFieldset
 {
   global $min_delete_ahead_secs, $max_delete_ahead_secs,
          $min_delete_ahead_enabled, $max_delete_ahead_enabled;
@@ -375,7 +381,7 @@ function get_fieldset_delete_ahead()
 }
 
 
-function get_fieldset_max_number()
+function get_fieldset_max_number() : ElementFieldset
 {
   global $interval_types,
          $max_per_interval_area_enabled, $max_per_interval_global_enabled,
@@ -406,14 +412,14 @@ function get_fieldset_max_number()
     $field = new FieldDiv;
 
     $checkbox_area = new ElementInputCheckbox();
-    $checkbox_area->setAttributes(array('name'  => "area_max_per_${interval_type}_enabled",
-                                        'id'    => "area_max_per_${interval_type}_enabled",
+    $checkbox_area->setAttributes(array('name'  => "area_max_per_{$interval_type}_enabled",
+                                        'id'    => "area_max_per_{$interval_type}_enabled",
                                         'class' => 'enabler'))
                   ->setChecked($max_per_interval_area_enabled[$interval_type]);
 
     $number_area = new ElementInputNumber();
     $number_area->setAttributes(array('min'   => '0',
-                                      'name'  => "area_max_per_${interval_type}",
+                                      'name'  => "area_max_per_$interval_type",
                                       'value' => $max_per_interval_area[$interval_type]));
 
     // Wrap the area and global controls in <div>s.  It'll make the CSS easier.
@@ -435,7 +441,7 @@ function get_fieldset_max_number()
     $div_global->addElement($checkbox_global)
                ->addElement($number_global);
 
-    $field->setLabel(get_vocab("max_per_${interval_type}"))
+    $field->setLabel(get_vocab("max_per_$interval_type"))
           ->addControlElement($div_area)
           ->addControlElement($div_global);
 
@@ -446,7 +452,7 @@ function get_fieldset_max_number()
 }
 
 
-function get_fieldset_max_secs()
+function get_fieldset_max_secs() : ElementFieldset
 {
   global $interval_types,
          $max_secs_per_interval_area_enabled, $max_secs_per_interval_global_enabled,
@@ -483,8 +489,8 @@ function get_fieldset_max_secs()
     $field = new FieldDiv;
 
     $checkbox_area = new ElementInputCheckbox();
-    $checkbox_area->setAttributes(array('name'  => "area_max_secs_per_${interval_type}_enabled",
-                                        'id'    => "area_max_secs_per_${interval_type}_enabled",
+    $checkbox_area->setAttributes(array('name'  => "area_max_secs_per_{$interval_type}_enabled",
+                                        'id'    => "area_max_secs_per_{$interval_type}_enabled",
                                         'class' => 'enabler'))
                   ->setChecked($max_secs_per_interval_area_enabled[$interval_type]);
 
@@ -493,12 +499,12 @@ function get_fieldset_max_secs()
     $options = Form::getTimeUnitOptions($max_unit);
 
     $select = new ElementSelect();
-    $select->setAttribute('name', "area_max_secs_per_${interval_type}_units")
+    $select->setAttribute('name', "area_max_secs_per_{$interval_type}_units")
            ->addSelectOptions($options, array_search($units, $options), true);
 
     $time_area = new ElementInputNumber();
     $time_area->setAttributes(array('min'   => '0',
-                                    'name'  => "area_max_secs_per_${interval_type}",
+                                    'name'  => "area_max_secs_per_$interval_type",
                                     'value' => $max));
 
     // Wrap the area and global controls in <div>s.  It'll make the CSS easier.
@@ -529,7 +535,7 @@ function get_fieldset_max_secs()
                ->addElement($time_global)
                ->addElement($select);
 
-    $field->setLabel(get_vocab("max_secs_per_${interval_type}"))
+    $field->setLabel(get_vocab("max_secs_per_$interval_type"))
           ->addControlElement($div_area)
           ->addControlElement($div_global);
 
@@ -540,7 +546,7 @@ function get_fieldset_max_secs()
 }
 
 
-function get_fieldset_max_duration()
+function get_fieldset_max_duration() : ElementFieldset
 {
   global $max_duration_enabled, $max_duration_secs, $max_duration_periods;
 
@@ -584,7 +590,7 @@ function get_fieldset_max_duration()
 }
 
 
-function get_fieldset_booking_policies()
+function get_fieldset_booking_policies() : ElementFieldset
 {
   global $enable_periods;
 
@@ -612,7 +618,7 @@ function get_fieldset_booking_policies()
 }
 
 
-function get_fieldset_confirmation_settings()
+function get_fieldset_confirmation_settings() : ElementFieldset
 {
   global $confirmation_enabled, $confirmed_default;
 
@@ -639,7 +645,7 @@ function get_fieldset_confirmation_settings()
 }
 
 
-function get_fieldset_approval_settings()
+function get_fieldset_approval_settings() : ElementFieldset
 {
   global $approval_enabled, $reminders_enabled;
 
@@ -664,7 +670,7 @@ function get_fieldset_approval_settings()
 }
 
 
-function get_fieldset_privacy_settings()
+function get_fieldset_privacy_settings() : ElementFieldset
 {
   global $private_enabled, $private_mandatory, $private_default;
 
@@ -698,7 +704,7 @@ function get_fieldset_privacy_settings()
 }
 
 
-function get_fieldset_privacy_display()
+function get_fieldset_privacy_display() : ElementFieldset
 {
   global $private_override;
 
@@ -721,7 +727,7 @@ function get_fieldset_privacy_display()
 }
 
 
-function get_fieldset_submit_buttons()
+function get_fieldset_submit_buttons() : ElementFieldset
 {
   $fieldset = new ElementFieldset();
 
@@ -729,11 +735,11 @@ function get_fieldset_submit_buttons()
   $field = new FieldInputSubmit();
 
   $back = new ElementInputSubmit();
-  $back->setAttributes(array('value'      => get_vocab('backadmin'),
+  $back->setAttributes(array('value'      => get_vocab('back'),
                              'formaction' => multisite('admin.php')));
   $field->addLabelClass('no_suffix')
         ->addLabelElement($back)
-        ->setControlAttribute('value', get_vocab('change'));
+        ->setControlAttribute('value', get_vocab('save'));
   $fieldset->addElement($field);
 
   return $fieldset;
@@ -749,8 +755,8 @@ $context = array(
   'year'      => $year,
   'month'     => $month,
   'day'       => $day,
-  'area'      => isset($area) ? $area : null,
-  'room'      => isset($room) ? $room : null
+  'area'      => $area ?? null,
+  'room'      => $room ?? null
 );
 
 print_header($context);
@@ -758,18 +764,17 @@ print_header($context);
 // Get the details for this area
 if (!isset($area) || is_null($data = get_area_details($area)))
 {
-  fatal_error(get_vocab('invalid_area'));
+  Errors::fatalError(get_vocab('invalid_area'));
 }
 
 $errors = get_form_var('errors', 'array');
 
 // Generate the form
-$form = new Form();
+$form = new Form(Form::METHOD_POST);
 
 $attributes = array('id'     => 'edit_area',
                     'class'  => 'standard',
-                    'action' => multisite('edit_area_handler.php'),
-                    'method' => 'post');
+                    'action' => multisite('edit_area_handler.php'));
 
 $form->setAttributes($attributes)
      ->addHiddenInput('area', $area);
