@@ -1,4 +1,5 @@
 <?php
+declare(strict_types=1);
 namespace MRBS;
 
 // This file contains internal configuration settings and checking.   You should not
@@ -15,10 +16,10 @@ namespace MRBS;
 // Variables no longer used in versions of MRBS > 1.4.4.1
 if (isset($provisional_enabled))
 {
-  $message = 'Please check your config file.   The variable $provisional_enabled ' .
+  $message = 'Please check your MRBS config file.   The variable $provisional_enabled ' .
              'is no longer used and has been replaced by $approval_enabled.';
   trigger_error($message, E_USER_NOTICE);
-  $approval_enabled = ($provisional_enabled) ? TRUE : FALSE;
+  $approval_enabled = (bool) $provisional_enabled;
 }
 
 // Variables no longer used in versions of MRBS > 1.4.5
@@ -27,47 +28,47 @@ if (isset($mail_settings['admin_all']))
   // We won't set $mail_settings['on_new'] because the default is TRUE
   // which gives the same behaviour as before, and if it's been set to FALSE
   // it means the site admin has deliberately changed it.
-  $message = 'Please check your config file.   The variable $mail_settings["admin_all"] ' .
+  $message = 'Please check your MRBS config file.   The variable $mail_settings["admin_all"] ' .
              'is no longer used and has been replaced by $mail_settings["on_new"], ' .
              '$mail_settings["on_change"] and $mail_settings["on_delete"].';
   trigger_error($message, E_USER_NOTICE);
-  $mail_settings['on_change'] = ($mail_settings['admin_all']) ? TRUE : FALSE;
+  $mail_settings['on_change'] = (bool) $mail_settings['admin_all'];
 }
 
 if (isset($mail_settings['admin_on_delete']))
 {
-  $message = 'Please check your config file.   The variable $mail_settings["admin_on_delete"] ' .
+  $message = 'Please check your MRBS config file.   The variable $mail_settings["admin_on_delete"] ' .
              'is no longer used and has been replaced by $mail_settings["on_delete"].';
   trigger_error($message, E_USER_NOTICE);
-  $mail_settings['on_delete'] = ($mail_settings['admin_on_delete']) ? TRUE : FALSE;
+  $mail_settings['on_delete'] = (bool) $mail_settings['admin_on_delete'];
 }
 
-if (!empty($dateformat))
+if (isset($dateformat))
 {
-  $message = 'Please check your config file.   The variable $dateformat ' .
-             'is no longer used and has been replaced by $strftime_format["daymonth"].';
-  trigger_error($message, E_USER_WARNING);
-  $strftime_format['daymonth']     = "%d %b";
+  $message = 'Please check your MRBS config file. The setting $dateformat ' .
+             'is no longer used and has been replaced by $datetime_formats. ' .
+             'See systemdefaults.inc.php for more details.';
+  trigger_error($message, E_USER_NOTICE);
 }
 
 // Variables no longer used in versions of MRBS > 1.4.7
 if (isset($highlight_method))
 {
-  $message = 'Please check your config file.   The variable $highlight_method ' .
+  $message = 'Please check your MRBS config file.   The variable $highlight_method ' .
              'is no longer used and is redundant.';
   trigger_error($message, E_USER_NOTICE);
 }
 
 if (isset($javascript_cursor))
 {
-  $message = 'Please check your config file.   The variable $javascript_cursor ' .
+  $message = 'Please check your MRBS config file.   The variable $javascript_cursor ' .
              'is no longer used and is redundant.';
   trigger_error($message, E_USER_NOTICE);
 }
 
 if (isset($mail_charset))
 {
-  $message = 'Please check your config file.   The variable $mail_charset ' .
+  $message = 'Please check your MRBS config file.   The variable $mail_charset ' .
              'is no longer used.   All emails are sent as UTF-8.';
   trigger_error($message, E_USER_NOTICE);
 }
@@ -75,27 +76,27 @@ if (isset($mail_charset))
 // Variables no longer used in versions of MRBS > 1.4.11
 if (isset($min_book_ahead_enabled))
 {
-  $message = 'Please check your config file.   The variable $min_book_ahead_enabled ' .
+  $message = 'Please check your MRBS config file.   The variable $min_book_ahead_enabled ' .
              'is no longer used and has been replaced by $min_create_ahead_enabled ' .
              'and $min_delete_ahead_enabled.';
   trigger_error($message, E_USER_WARNING);
-  $min_create_ahead_enabled = ($min_book_ahead_enabled) ? TRUE : FALSE;
-  $min_delete_ahead_enabled = ($min_book_ahead_enabled) ? TRUE : FALSE;
+  $min_create_ahead_enabled = (bool) $min_book_ahead_enabled;
+  $min_delete_ahead_enabled = (bool) $min_book_ahead_enabled;
 }
 
 if (isset($max_book_ahead_enabled))
 {
-  $message = 'Please check your config file.   The variable $max_book_ahead_enabled ' .
+  $message = 'Please check your MRBS config file.   The variable $max_book_ahead_enabled ' .
              'is no longer used and has been replaced by $max_create_ahead_enabled ' .
              'and $max_delete_ahead_enabled.';
   trigger_error($message, E_USER_WARNING);
-  $max_create_ahead_enabled = ($max_book_ahead_enabled) ? TRUE : FALSE;
+  $max_create_ahead_enabled = (bool) $max_book_ahead_enabled;
   // No need to do anything about $max_delete_ahead_enabled as it didn't apply in the old system
 }
 
 if (isset($min_book_ahead_secs))
 {
-  $message = 'Please check your config file.   The variable $min_book_ahead_secs ' .
+  $message = 'Please check your MRBS config file.   The variable $min_book_ahead_secs ' .
              'is no longer used and has been replaced by $min_create_ahead_secs ' .
              'and $min_delete_ahead_secs.';
   trigger_error($message, E_USER_WARNING);
@@ -105,7 +106,7 @@ if (isset($min_book_ahead_secs))
 
 if (isset($max_book_ahead_secs))
 {
-  $message = 'Please check your config file.   The variable $max_book_ahead_secs ' .
+  $message = 'Please check your MRBS config file.   The variable $max_book_ahead_secs ' .
              'is no longer used and has been replaced by $max_create_ahead_secs ' .
              'and $max_delete_ahead_secs.';
   trigger_error($message, E_USER_WARNING);
@@ -115,7 +116,7 @@ if (isset($max_book_ahead_secs))
 
 if (isset($maxlength))
 {
-  $message = 'Please check your config file.   The variable $maxlength ' .
+  $message = 'Please check your MRBS config file.   The variable $maxlength ' .
              'is no longer used and maximum field lengths are now calculated automatically.';
   trigger_error($message, E_USER_NOTICE);
 }
@@ -124,7 +125,7 @@ if (isset($maxlength))
 if (isset($db_nopersist))
 {
   $db_persist = !$db_nopersist;
-  $message = 'Please check your config file.  The $db_nopersist config variable ' .
+  $message = 'Please check your MRBS config file.  The $db_nopersist config variable ' .
              'has been replaced by $db_persist';
   trigger_error($message, E_USER_NOTICE);
 }
@@ -133,7 +134,7 @@ if (isset($db_nopersist))
 if (false !== ($key = array_search('start_date', $edit_entry_field_order)))
 {
   $edit_entry_field_order[$key] = 'start_time';
-  $message = 'Please check your config file.  The value \'start_date\' in the variable ' .
+  $message = 'Please check your MRBS config file.  The value \'start_date\' in the variable ' .
              '$edit_entry_field_order has been replaced by \'start_time\'.';
   trigger_error($message, E_USER_NOTICE);
 }
@@ -141,7 +142,7 @@ if (false !== ($key = array_search('start_date', $edit_entry_field_order)))
 if (false !== ($key = array_search('end_date', $edit_entry_field_order)))
 {
   $edit_entry_field_order[$key] = 'end_time';
-  $message = 'Please check your config file.  The value \'end_date\' in the variable ' .
+  $message = 'Please check your MRBS config file.  The value \'end_date\' in the variable ' .
              '$edit_entry_field_order has been replaced by \'end_time\'.';
   trigger_error($message, E_USER_NOTICE);
 }
@@ -149,7 +150,7 @@ if (false !== ($key = array_search('end_date', $edit_entry_field_order)))
 if (false !== ($key = array_search('areas', $edit_entry_field_order)))
 {
   $edit_entry_field_order[$key] = 'room_id';
-  $message = 'Please check your config file.  The value \'areas\' in the variable ' .
+  $message = 'Please check your MRBS config file.  The value \'areas\' in the variable ' .
              '$edit_entry_field_order has been replaced by \'room_id\'.';
   trigger_error($message, E_USER_NOTICE);
 }
@@ -164,11 +165,28 @@ if (false !== ($key = array_search('rooms', $edit_entry_field_order)))
   {
     $edit_entry_field_order[$key] = 'room_id';
   }
-  $message = 'Please check your config file.  The value \'rooms\' in the variable ' .
+  $message = 'Please check your MRBS config file.  The value \'rooms\' in the variable ' .
              '$edit_entry_field_order has been replaced by \'room_id\'.';
   trigger_error($message, E_USER_NOTICE);
 }
 
+// Variables no longer used in versions of MRBS > 1.11.0
+if (isset($twentyfourhour_format))
+{
+  $message = 'Please check your MRBS config file. The setting $twentyfourhour_format ' .
+             'is no longer used and whether a 12 or 24-hour clock is used is determined ' .
+             'by the locale. See systemdefaults.inc.php for more details.';
+  trigger_error($message, E_USER_NOTICE);
+}
+
+if (isset($strftime_format))
+{
+  $message = 'Please check your MRBS config file. The setting $strftime_format ' .
+             'is no longer used and has been replaced by $datetime_formats. ' .
+             'See systemdefaults.inc.php for more details.';
+  trigger_error($message, E_USER_NOTICE);
+}
+
 
 /********************************************************
  * Checking
@@ -185,22 +203,22 @@ if ($enable_periods)
 {
   if (isset($periods) && (count($periods) > 60))
   {
-    die('Configuration error: too many periods defined');
+    die('MRBS configuration error: too many periods defined');
   }
 }
 else
 {
   if (!isset($resolution))
   {
-    die('Configuration error: $resolution has not been set.');
+    die('MRBS configuration error: $resolution has not been set.');
   }
   if ($resolution <= 0)
   {
-    die('Configuration error: $resolution is less than or equal to zero.');
+    die('MRBS configuration error: $resolution is less than or equal to zero.');
   }
   if ($resolution%60 != 0)
   {
-    die('Configuration error: $resolution is not an integral number of minutes.');
+    die('MRBS configuration error: $resolution is not an integral number of minutes.');
   }
   // Not safe to call get_start_first_slot() etc. here as the timezone won't necessarily have
   // been set yet(although quite often it will have been by php.ini using date.timezone)
@@ -213,7 +231,7 @@ else
   $start_difference = $start_last_slot - $start_first_slot;    // seconds
   if ($start_difference%$resolution != 0)
   {
-    die('Configuration error: make sure that the length of the booking day is an integral multiple of $resolution.');
+    die('MRBS configuration error: make sure that the length of the booking day is an integral multiple of $resolution.');
   }
 }
 
@@ -224,20 +242,17 @@ else
 
  define('DOCTYPE', '<!DOCTYPE html>');
 
- // Records which DOCTYPE is being used.    Do not change - it will not change the DOCTYPE
- // that is used;  it is merely used when the code needs to know the DOCTYPE, for example
- // in calls to nl2br.   TRUE means XHTML, FALSE means HTML.
- define('IS_XHTML', FALSE);
-
 
 /*************************************************
  * General constants - internal use, do not change
  *************************************************/
 
-define('DAYS_PER_WEEK',    7);
-define('MINUTES_PER_DAY',  24*60);
-define('SECONDS_PER_DAY',  MINUTES_PER_DAY * 60);
-define('SECONDS_PER_HOUR', 3600);
+define('DAYS_PER_WEEK',       7);
+define('MONTHS_PER_YEAR',     12);
+define('MINUTES_PER_DAY',     24*60);
+define('SECONDS_PER_DAY',     MINUTES_PER_DAY * 60);
+define('SECONDS_PER_HOUR',    3600);
+define('SECONDS_PER_MINUTE',  60);
 
 /*************************************************
  * REPORT constants - internal use, do not change
@@ -252,6 +267,10 @@ define('OUTPUT_HTML',  0);
 define('OUTPUT_CSV',   1);
 define('OUTPUT_ICAL',  2);
 
+// Fallback values
+define('FALLBACK_SORTBY', 'r');
+define('FALLBACK_SUMBY',  'c');
+
 // Constants for matching boolean fields
 define('BOOLEAN_MATCH_FALSE', 0);
 define('BOOLEAN_MATCH_TRUE',  1);
@@ -310,20 +329,6 @@ define('STATUS_AWAITING_APPROVAL', 0x02);
 define('STATUS_TENTATIVE',         0x04);
 
 
-/*************************************************
- * REPEAT TYPE CODES - internal use, do not change
- *************************************************/
-
-define('REP_NONE',            0);
-define('REP_DAILY',           1);
-define('REP_WEEKLY',          2);
-define('REP_MONTHLY',         3);
-define('REP_YEARLY',          4);
-
-define('REP_MONTH_ABSOLUTE', 0);
-define('REP_MONTH_RELATIVE', 1);
-
-
 /*************************************************
  * DIRECTORIES - internal use, do not change
  *************************************************/
@@ -332,18 +337,6 @@ define('TZDIR',         'tzurl/zoneinfo');          // Directory containing TZUR
 define('TZDIR_OUTLOOK', 'tzurl/zoneinfo-outlook');  // Outlook compatible TZURL definitions
 
 
-/*****************************************
- * ICALENDAR - internal use, do not change
- *****************************************/
-
-define ('RFC5545_FORMAT', 'Ymd\THis');  // Format for expressing iCalendar dates
-define ('ICAL_EOL', "\r\n");            // Lines must be terminated by CRLF
-
-// Create an array which can be used to map day of the week numbers (0..6)
-// onto days of the week as defined in RFC 5545
-$RFC_5545_days = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
-
-
 /****************************************************************
  * DATABASE TABLES  - internal use, do not change
  ****************************************************************/
@@ -524,20 +517,5 @@ $area_defaults['max_secs_per_year']             = $max_secs_per_interval_area['y
 $area_defaults['max_secs_per_future_enabled']   = $max_secs_per_interval_area_enabled['future'];
 $area_defaults['max_secs_per_future']           = $max_secs_per_interval_area['future'];
 
-
-// We send Ajax requests to ajax/del_entries.php with data as an array of ids.
-// In order to stop the POST request getting too large and triggering a 406
-// error, we split the requests into batches with a maximum number of ids
-// in the array defined below.
-define('DEL_ENTRIES_AJAX_BATCH_SIZE', 100);
-
 // Interval types used in booking policies
 $interval_types = array('day', 'week', 'month', 'year', 'future');
-
-
-/********************************************************
- * JavaScript - internal use, do not change
- ********************************************************/
-
-// Puts Javascript into strict mode
-$use_strict = true;
