@@ -4,18 +4,13 @@ namespace MRBS\Auth;
 
 class AuthConfig extends Auth
 {
-  /* authValidateUser($user, $pass)
-   *
-   * Checks if the specified username/password pair are valid
-   *
-   * $user  - The user name
-   * $pass  - The password
-   *
-   * Returns:
-   *   false    - The pair are invalid or do not exist
-   *   string   - The validated username
+  /**
    */
-  public function validateUser($user, $pass)
+  public function validateUser(
+    #[\SensitiveParameter]
+    ?string $user,
+    #[\SensitiveParameter]
+    ?string $pass)
   {
     global $auth;
 
@@ -28,8 +23,8 @@ class AuthConfig extends Auth
     if ((isset($auth["user"][$user]) &&
         ($auth["user"][$user] == $pass)
       ) ||
-      (isset($auth["user"][\MRBS\utf8_strtolower($user)]) &&
-        ($auth["user"][\MRBS\utf8_strtolower($user)] == $pass)
+      (isset($auth["user"][mb_strtolower($user)]) &&
+        ($auth["user"][mb_strtolower($user)] == $pass)
       ))
     {
       return $user;    // User validated
@@ -40,7 +35,7 @@ class AuthConfig extends Auth
 
 
   // Return an array of users, indexed by 'username' and 'display_name'
-  public function getUsernames()
+  public function getUsernames() : array
   {
     global $auth;
 
